/**
 * 
 */
package com.wjholden.kurosuke;

import java.applet.Applet;
import java.awt.AWTException;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Robot;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.io.StringReader;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;

import javax.swing.JTextArea;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import netscape.javascript.JSObject;

import org.w3c.dom.CharacterData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author John
 *
 */
public class KurosukeControllerApplet extends Applet {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7283063353587085713L;
	private static final int PORT = 5683;
	private static final String IP_GROUP = "239.56.8.3";
	
	public static String text;
	public static String error;
	public static boolean closing = false;
	
	private NetworkLogic nl;
	private JTextArea textArea;
	
	public void init ()
	{
		try {
			nl = new NetworkLogic();
			nl.start();
		} catch (IOException e) {
			KurosukeControllerApplet.text = e.toString();
		} catch (IllegalThreadStateException e) {
			KurosukeControllerApplet.text = e.toString();
		}
		
		textArea = new JTextArea(null, 20, 80);
		textArea.setFont(new Font(Font.MONOSPACED, Font.PLAIN, 12));
		
		text = "No input.";
	}
	
	public void stop ()
	{
		closing = true;
	}

	public void paint (Graphics g)
	{
		String temp = "";
		if (text != null)
			temp += text;
		if (error != null)
			temp += "\nError: " + error;
			
		textArea.setText(temp);
		this.add(textArea);
		
		error = "";
		
		JSObject.getWindow(this).call("showXmlOutput", new Object[] { temp });
	}
	
	private class NetworkLogic extends Thread
	{
		private InetAddress group;
		private MulticastSocket socket;
		private DatagramPacket packet;
		private Robot robbie;
		
		public NetworkLogic() throws IOException
		{
			group = InetAddress.getByName(IP_GROUP);
			socket = new MulticastSocket(PORT);
			socket.joinGroup(group);
			
			try {
				robbie = new Robot();
			} catch (AWTException e) {
				KurosukeControllerApplet.error = e.toString();
				closing = true;
			}
		}
		
		public void run ()
		{
			while (!KurosukeControllerApplet.closing)
			{
				byte[] buf = new byte[8192];
				packet = new DatagramPacket(buf, buf.length);
				try {
					socket.receive(packet);
				} catch (IOException e) {
					KurosukeControllerApplet.text = e.toString();
				}

				String s = new String(packet.getData());
				s = s.trim();
				parseXml(s);
				setText(s);
			}
			
			try {
				socket.leaveGroup(group);
			} catch (IOException e) {
				KurosukeControllerApplet.error = e.toString();
			}
		}
		
		private void parseXml (String s)
		{
			try {
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db;
				db = dbf.newDocumentBuilder();
				InputSource is = new InputSource();
				is.setCharacterStream(new StringReader(s));
				
				Document dom = db.parse(is);
				
				parseDocument(dom);
			} catch (ParserConfigurationException e) {
				KurosukeControllerApplet.error = e.toString();
				return;
			} catch (SAXException e) {
				KurosukeControllerApplet.error = e.toString();
				return;
			} catch (IOException e) {
				KurosukeControllerApplet.error = e.toString();
				return;
			}
		}
		
		private void parseDocument(Document d)
		{
			Element docElement = d.getDocumentElement();
			NodeList nl = docElement.getElementsByTagName("keyPress");
			if (nl != null && nl.getLength() > 0)
			{
				for (int i = 0; i < nl.getLength(); i++)
				{
					Element e = (Element) nl.item(i);
					parseKeyPress(e);
				}
			}
			else
			{
				setText("XML node list empty.");
			}
		}
		
		private void parseKeyPress (Element e)
		{
			int playerId, up, down, left, right, btnA, btnB;
			
			playerId = Integer.parseInt(e.getAttribute("playerId"));
			
			up = getIntegerValueFromElement(e, "up");
			down = getIntegerValueFromElement(e, "down");
			left = getIntegerValueFromElement(e, "left");
			right = getIntegerValueFromElement(e, "right");
			btnA = getIntegerValueFromElement(e, "btnA");
			btnB = getIntegerValueFromElement(e, "btnB");
			
			/*setText("playerId = " + playerId + "\n" +
					"up = " + up + "\n" +
					"down = " + down + "\n" +
					"left = " + left + "\n" +
					"right = " + right + "\n" +
					"btnA = " + btnA + "\n" +
					"btnB = " + btnB + "\n");*/
			try {
				simulateInput(playerId, up, down, left, right, btnA, btnB);
			} catch (Exception e1) {
				KurosukeControllerApplet.error = e1.toString();
			}
		}
		
		private int getIntegerValueFromElement (Element e, String tagname)
		{
			NodeList name = e.getElementsByTagName(tagname);
			Element line = (Element) name.item(0);
			Node child = line.getFirstChild();
			if (child instanceof CharacterData)
			{
				try
				{
				return(Integer.parseInt(((CharacterData) child).getData()));
				}
				catch (NumberFormatException ex)
				{
					KurosukeControllerApplet.error = ex.toString();
				}
			}
			return(-1);
		}
		
		private void setText (String s)
		{
			KurosukeControllerApplet.text = s;
			repaint();
		}
		
		/**
		 * 
		 * @param playerId
		 * @param up
		 * @param down
		 * @param left
		 * @param right
		 * @param btnA
		 * @param btnB Not used.
		 */
		private void simulateInput(int playerId, int up, int down, int left, int right, int btnA, int btnB) throws IllegalArgumentException, AWTException
		{
			switch (playerId)
			{
			case 1:
				if (up == 1) robbie.keyPress(KeyEvent.VK_UP); else robbie.keyRelease(KeyEvent.VK_UP);
				if (down == 1) robbie.keyPress(KeyEvent.VK_DOWN); else robbie.keyRelease(KeyEvent.VK_DOWN);
				if (left == 1) robbie.keyPress(KeyEvent.VK_LEFT); else robbie.keyRelease(KeyEvent.VK_LEFT);
				if (right == 1) robbie.keyPress(KeyEvent.VK_RIGHT); else robbie.keyRelease(KeyEvent.VK_RIGHT);
				// looks different because "buttons" need to be tapped then immediately released
				if (btnA == 1) { robbie.keyPress(KeyEvent.VK_CONTROL); robbie.keyRelease(KeyEvent.VK_CONTROL); } 
				break;
			case 2:
				if (up == 1) robbie.keyPress(KeyEvent.VK_W); else robbie.keyRelease(KeyEvent.VK_W);
				if (down == 1) robbie.keyPress(KeyEvent.VK_S); else robbie.keyRelease(KeyEvent.VK_S);
				if (left == 1) robbie.keyPress(KeyEvent.VK_A); else robbie.keyRelease(KeyEvent.VK_A);
				if (right == 1) robbie.keyPress(KeyEvent.VK_D); else robbie.keyRelease(KeyEvent.VK_D);
				if (btnA == 1) { robbie.keyPress(KeyEvent.VK_SPACE); robbie.keyRelease(KeyEvent.VK_SPACE); } 
				break;
			case 3:
				if (up == 1) robbie.keyPress(KeyEvent.VK_I); else robbie.keyRelease(KeyEvent.VK_I);
				if (down == 1) robbie.keyPress(KeyEvent.VK_K); else robbie.keyRelease(KeyEvent.VK_K);
				if (left == 1) robbie.keyPress(KeyEvent.VK_J); else robbie.keyRelease(KeyEvent.VK_J);
				if (right == 1) robbie.keyPress(KeyEvent.VK_L); else robbie.keyRelease(KeyEvent.VK_L);
				if (btnA == 1) { robbie.keyPress(KeyEvent.VK_SHIFT); robbie.keyRelease(KeyEvent.VK_SHIFT); } 
				break;
			case 4:
				if (up == 1) robbie.keyPress(KeyEvent.VK_8); else robbie.keyRelease(KeyEvent.VK_8);
				if (down == 1) robbie.keyPress(KeyEvent.VK_2); else robbie.keyRelease(KeyEvent.VK_2);
				if (left == 1) robbie.keyPress(KeyEvent.VK_4); else robbie.keyRelease(KeyEvent.VK_4);
				if (right == 1) robbie.keyPress(KeyEvent.VK_6); else robbie.keyRelease(KeyEvent.VK_6);
				if (btnA == 1) { robbie.keyPress(KeyEvent.VK_PLUS); robbie.keyRelease(KeyEvent.VK_PLUS); } 
				break;
			}
		}
	}
}
